package it.amattioli.encapsulate.dates.specifications;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Date;

import org.apache.commons.beanutils.PropertyUtils;

import it.amattioli.dominate.Entity;
import it.amattioli.dominate.specifications.ChainedSpecification;
import it.amattioli.dominate.specifications.SpecificationsConfig;
import it.amattioli.encapsulate.dates.TimeInterval;

public abstract class DateSpecification<T extends Entity<?>> extends ChainedSpecification<T> {
	private static final String DATE_SPECIFICATION_CONFIG = "/it/amattioli/encapsulate/dates/specifications/dateSpecifications.properties";
	
	static {
		SpecificationsConfig.addSpecificationConfig(DATE_SPECIFICATION_CONFIG);
	}
	
	private String propertyName;
	private TimeInterval value;
	
	public static <T extends Entity<?>> DateSpecification<T> newInstance(String propertyName) {
		DateSpecification<T> result = null;
		for (Class<? extends DateSpecification> curr: SpecificationsConfig.getSpecificationClasses(DateSpecification.class)) {
			Constructor<? extends DateSpecification> c;
			try {
				c = curr.getConstructor(String.class, DateSpecification.class);
			} catch (SecurityException e) {
				throw new RuntimeException(e);
			} catch (NoSuchMethodException e) {
				throw new RuntimeException(e);
			}
			try {
				result = (DateSpecification<T>)c.newInstance(propertyName, result);
			} catch (IllegalArgumentException e) {
				throw new RuntimeException(e);
			} catch (InstantiationException e) {
				throw new RuntimeException(e);
			} catch (IllegalAccessException e) {
				throw new RuntimeException(e);
			} catch (InvocationTargetException e) {
				throw new RuntimeException(e);
			}
		}
		return result;
	}
	
	public DateSpecification(String propertyName) {
		this.propertyName = propertyName;
	}
	
	public DateSpecification(String propertyName, DateSpecification<T> chained) {
		super(chained);
		this.propertyName = propertyName;
	}

	protected String getPropertyName() {
		return propertyName;
	}
	
	public void setValue(TimeInterval value) {
		this.value = value;
		if (getNextInChain() != null) {
			((DateSpecification<T>)getNextInChain()).setValue(value);
		}
	}

	public TimeInterval getValue() {
		return value;
	}
	
	@Override
	public boolean isSatisfiedBy(T entity) {
		if (!wasSet()) {
			return isSatisfiedIfNotSet();
		}
		try {
			Date propertyValue = (Date)PropertyUtils.getProperty(entity, getPropertyName());
			return getValue().includes(propertyValue);
		} catch (IllegalAccessException e) {
			throw new RuntimeException();
		} catch (InvocationTargetException e) {
			throw new RuntimeException();
		} catch (NoSuchMethodException e) {
			throw new RuntimeException();
		}
	}
	
	@Override
	public boolean wasSet() {
		return getValue() != null;
	}
}
