package it.amattioli.encapsulate.money.specifications;

import it.amattioli.dominate.Entity;
import it.amattioli.dominate.specifications.ChainedSpecification;
import it.amattioli.dominate.specifications.SpecificationsConfig;
import it.amattioli.encapsulate.money.Money;
import it.amattioli.encapsulate.range.Range;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import org.apache.commons.beanutils.PropertyUtils;

public abstract class MoneySpecification<T extends Entity<?>> extends ChainedSpecification<T> {
	private static final String MONEY_SPECIFICATION_CONFIG = "/it/amattioli/encapsulate/money/specifications/moneySpecifications.properties";
	
	static {
		SpecificationsConfig.addSpecificationConfig(MONEY_SPECIFICATION_CONFIG);
	}
	
	private String propertyName;
	private Range<Money> value;
	
	public static <T extends Entity<?>> MoneySpecification<T> newInstance(String propertyName) {
		MoneySpecification<T> result = null;
		for (Class<? extends MoneySpecification> curr: SpecificationsConfig.getSpecificationClasses(MoneySpecification.class)) {
			Constructor<? extends MoneySpecification> c;
			try {
				c = curr.getConstructor(String.class, MoneySpecification.class);
			} catch (SecurityException e) {
				throw new RuntimeException(e);
			} catch (NoSuchMethodException e) {
				throw new RuntimeException(e);
			}
			try {
				result = (MoneySpecification<T>)c.newInstance(propertyName, result);
			} catch (IllegalArgumentException e) {
				throw new RuntimeException(e);
			} catch (InstantiationException e) {
				throw new RuntimeException(e);
			} catch (IllegalAccessException e) {
				throw new RuntimeException(e);
			} catch (InvocationTargetException e) {
				throw new RuntimeException(e);
			}
		}
		return result;
	}
	
	public MoneySpecification(String propertyName) {
		this.propertyName = propertyName;
	}
	
	public MoneySpecification(String propertyName, MoneySpecification<T> chained) {
		super(chained);
		this.propertyName = propertyName;
	}

	protected String getPropertyName() {
		return propertyName;
	}
	
	public void setValue(Range<Money> value) {
		this.value = value;
		if (getNextInChain() != null) {
			((MoneySpecification<T>)getNextInChain()).setValue(value);
		}
	}

	public Range<Money> getValue() {
		return value;
	}
	
	@Override
	public boolean isSatisfiedBy(T entity) {
		if (!wasSet()) {
			return isSatisfiedIfNotSet();
		}
		try {
			Money propertyValue = (Money)PropertyUtils.getProperty(entity, getPropertyName());
			return getValue().includes(propertyValue);
		} catch (IllegalAccessException e) {
			throw new RuntimeException();
		} catch (InvocationTargetException e) {
			throw new RuntimeException();
		} catch (NoSuchMethodException e) {
			throw new RuntimeException();
		}
	}
	
	@Override
	public boolean wasSet() {
		return getValue() != null;
	}
}

